@@ -1,3 +1,5 @@
+"""Configuration of the viewer"""
+
 #
 # Copyright 2023 Bernhard Walter
 #
@@ -16,7 +18,7 @@
 
 from enum import Enum
 
-from .comms import send_command, send_config, get_port
+from .comms import send_command, send_config, get_port, is_pytest
 
 __all__ = [
     "workspace_config",
@@ -34,18 +36,24 @@ __all__ = [
 
 
 class Camera(Enum):
+    """Camera reset modes"""
+
     RESET = "reset"
     CENTER = "center"
     KEEP = "keep"
 
 
 class Collapse(Enum):
+    """Collapse modes for the CAD navigation tree"""
+
     NONE = 0
     LEAVES = 1
     ALL = 2
     ROOT = 3
 
 
+COLLAPSE_MAPPING = ["E", "1", "C", "R"]
+
 CONFIG_UI_KEYS = [
     "axes",
     "axes0",
@@ -58,6 +66,15 @@ CONFIG_UI_KEYS = [
     "direct_intensity",
     "metalness",
     "roughness",
+    "clip_slider_0",
+    "clip_slider_1",
+    "clip_slider_2",
+    "clip_normal_0",
+    "clip_normal_1",
+    "clip_normal_2",
+    "clip_planes",
+    "clip_intersection",
+    "clip_object_colors",
 ]
 
 CONFIG_WORKSPACE_KEYS = CONFIG_UI_KEYS + [
@@ -65,9 +82,9 @@ CONFIG_WORKSPACE_KEYS = CONFIG_UI_KEYS + [
     "collapse",
     "dark",
     "glass",
-    "measure_tools",
     "orbit_control",
     "ticks",
+    "center_grid",
     "tools",
     "tree_width",
     "up",
@@ -88,7 +105,6 @@ CONFIG_WORKSPACE_KEYS = CONFIG_UI_KEYS + [
     "default_vertexcolor",
     "default_opacity",
     "deviation",
-    "measure_tools",
     "modifier_keys",
 ]
 
@@ -101,6 +117,8 @@ CONFIG_CONTROL_KEYS = [
     "render_joints",
     "render_normals",
     "reset_camera",
+    "show_parent",
+    "show_sketch_local",
     "timeit",
 ]
 
@@ -131,6 +149,16 @@ CONFIG_SET_KEYS = [
     "tools",
     "tree_width",
     "collapse",
+    # "tab",
+    # "clip_slider_0",
+    # "clip_slider_1",
+    # "clip_slider_2",
+    # "clip_normal_0",
+    # "clip_normal_1",
+    # "clip_normal_2",
+    # "clip_intersection",
+    # "clip_planes",
+    # "clip_object_colors",
 ]
 
 DEFAULTS = {
@@ -139,12 +167,16 @@ DEFAULTS = {
     "render_mates": False,
     "render_joints": False,
     "helper_scale": 1.0,
+    "show_parent": False,
+    "show_sketch_local": True,
     "timeit": False,
     "reset_camera": Camera.RESET,
+    "collapse": Collapse.ROOT,
     "debug": False,
 }
 
 
+# pylint: disable=too-many-arguments,unused-argument,too-many-locals
 def set_viewer_config(
     axes=None,
     axes0=None,
@@ -172,27 +204,52 @@ def set_viewer_config(
     collapse=None,
     reset_camera=None,
     states=None,
+    tab=None,
+    clip_slider_0=None,
+    clip_slider_1=None,
+    clip_slider_2=None,
+    clip_normal_0=None,
+    clip_normal_1=None,
+    clip_normal_2=None,
+    clip_intersection=None,
+    clip_planes=None,
+    clip_object_colors=None,
+    port=None,
 ):
+    """Set viewer config"""
+    if port is None:
+        port = get_port()
+
     config = {k: v for k, v in locals().items() if v is not None}
+    if config.get("collapse") is not None:
+        config["collapse"] = COLLAPSE_MAPPING[config["collapse"].value]
     data = {
         "type": "ui",
         "config": config,
     }
-    send_config(data)
+
+    try:
+        send_config(data, port)
+
+    except Exception as ex:
+        raise RuntimeError(
+            "Cannot set viewer config. Is the viewer running?\n" + str(ex.args)
+        ) from ex
 
 
 def get_default(key):
+    """Get default value for key"""
     return DEFAULTS.get(key)
 
 
 def get_defaults():
+    """Get all defaults"""
     return DEFAULTS
 
 
 def set_defaults(
     glass=None,
     tools=None,
-    measure_tools=None,
     tree_width=None,
     axes=None,
     axes0=None,
@@ -204,10 +261,20 @@ def set_defaults(
     orbit_control=None,
     collapse=None,
     ticks=None,
+    center_grid=None,
     up=None,
     explode=None,
     zoom=None,
     reset_camera=None,
+    clip_slider_0=None,
+    clip_slider_1=None,
+    clip_slider_2=None,
+    clip_normal_0=None,
+    clip_normal_1=None,
+    clip_normal_2=None,
+    clip_intersection=None,
+    clip_planes=None,
+    clip_object_colors=None,
     pan_speed=None,
     rotate_speed=None,
     zoom_speed=None,
@@ -224,17 +291,19 @@ def set_defaults(
     render_normals=None,
     render_mates=None,
     render_joints=None,
+    show_parent=None,
+    show_sketch_local=None,
     helper_scale=None,
     mate_scale=None,  # DEPRECATED
     debug=None,
     timeit=None,
 ):
+    # pylint: disable=line-too-long
     """Set viewer defaults
     Keywords to configure the viewer:
     - UI
         glass:             Use glass mode where tree is an overlay over the cad object (default=False)
         tools:             Show tools (default=True)
-        measure_tools:     Show measure tools (default=False)
         tree_width:        Width of the object tree (default=240)
 
     - Viewer
@@ -250,8 +319,9 @@ def set_defaults(
                            Collapse.ROOT: expand root only,
                            Collapse.ALL: collapse all nodes,
                            Collapse.NONE: expand all nodes
-                           (default=Collapse.LEAVES)
+                           (default=Collapse.ROOT)
         ticks:             Hint for the number of ticks in both directions (default=10)
+        center_grid:       Center the grid at the origin or center of mass (default=False)
         up:                Use z-axis ('Z') or y-axis ('Y') as up direction for the camera (default="Z")
         explode:           Turn on explode mode (default=False)
 
@@ -263,6 +333,16 @@ def set_defaults(
                            Camera.CENTER: Keep camera position, rotation, toom, but look at center
                            Camera.KEEP: Keep camera position, rotation, toom, and target
                            (default=Camera.RESET)
+        clip_slider_0:     Setting of clipping slider 0 (default=None)
+        clip_slider_1:     Setting of clipping slider 1 (default=None)
+        clip_slider_2:     Setting of clipping slider 2 (default=None)
+        clip_normal_0:     Setting of clipping normal 0 (default=[-1,0,0])
+        clip_normal_1:     Setting of clipping normal 1 (default=[0,-1,0])
+        clip_normal_2:     Setting of clipping normal 2 (default=[0,0,-1])
+        clip_intersection: Use clipping intersection mode (default=[False])
+        clip_planes:       Show clipping plane helpers (default=False)
+        clip_object_colors: Use object color for clipping caps (default=False)
+
         pan_speed:         Speed of mouse panning (default=1)
         rotate_speed:      Speed of mouse rotate (default=1)
         zoom_speed:        Speed of mouse zoom (default=1)
@@ -283,6 +363,8 @@ def set_defaults(
         render_normals:    Render normals (default=False)
         render_mates:      Render mates for MAssemblies (default=False)
         render_joints:     Render mates for MAssemblies (default=False)
+        show_parent:       Render parent of faces, edges or vertices as wireframe (default=False)
+        show_sketch_local: In build123d show local sketch in addition to relocate sketch (default=True)
         helper_scale:      Scale of rendered helpers (locations, axis, mates for MAssemblies) (default=1)
 
     - Debug
@@ -294,7 +376,6 @@ def set_defaults(
 
     kwargs = check_deprecated(kwargs)
 
-    global DEFAULTS
     for key, value in kwargs.items():
         if key in CONFIG_KEYS:
             DEFAULTS[key] = value
@@ -305,14 +386,21 @@ def set_defaults(
 
 
 def preset(key, value):
+    """Set default value for key"""
     return get_default(key) if value is None else value
 
 
 def ui_filter(conf):
+    """Filter out all non-UI keys from the config dict"""
     return {k: v for k, v in conf.items() if k in CONFIG_UI_KEYS}
 
 
 def status(port=None, debug=False):
+    """Get viewer status"""
+
+    if is_pytest():
+        return {}
+
     if port is None:
         port = get_port()
     try:
@@ -320,15 +408,25 @@ def status(port=None, debug=False):
         if debug:
             return response.get("_debugStarted", False)
         else:
-            return response.get("text", {})
+            return dict(sorted(response.get("text", {}).items()))
 
     except Exception as ex:
         raise RuntimeError(
             "Cannot access viewer status. Is the viewer running?\n" + str(ex.args)
-        )
+        ) from ex
 
 
 def workspace_config(port=None):
+    """Get viewer workspace config"""
+
+    if is_pytest():
+        return {
+            "_splash": False,
+            "default_facecolor": (1, 234, 56),
+            "default_thickedgecolor": (123, 45, 6),
+            "default_vertexcolor": (123, 45, 6),
+        }
+
     if port is None:
         port = get_port()
     try:
@@ -343,16 +441,17 @@ def workspace_config(port=None):
             "C": Collapse.ALL,
             "R": Collapse.ROOT,
         }
-        conf["collapse"] = mapping[conf["collapse"]]
-        return conf
+        conf["collapse"] = mapping[conf.get("collapse", "R")]
+        return dict(sorted(conf.items()))
 
     except Exception as ex:
         raise RuntimeError(
             "Cannot access viewer config. Is the viewer running?\n" + str(ex.args)
-        )
+        ) from ex
 
 
 def combined_config(port=None, use_status=True):
+    """Get combined config from workspace and status"""
     if port is None:
         port = get_port()
 
@@ -363,7 +462,7 @@ def combined_config(port=None, use_status=True):
     except Exception as ex:
         raise RuntimeError(
             "Cannot access viewer config. Is the viewer running?\n" + str(ex.args)
-        )
+        ) from ex
 
     if use_status and wspace_config["_splash"]:
         del wspace_config["_splash"]
@@ -373,15 +472,15 @@ def combined_config(port=None, use_status=True):
         wspace_config["ortho"] = False
         wspace_config["transparent"] = False
         wspace_config["black_edges"] = False
-        wspace_config["measure_tools"] = False
 
     wspace_config.update(DEFAULTS)
     if use_status:
         wspace_config.update(ui_filter(wspace_status))
-    return wspace_config
+    return dict(sorted(wspace_config.items()))
 
 
 def get_changed_config(key=None):
+    """Get changed config from workspace and status"""
     wspace_config = workspace_config()
     wspace_config.update(DEFAULTS)
     if key is None:
@@ -392,7 +491,7 @@ def get_changed_config(key=None):
 
 def reset_defaults():
     """Reset defaults not given in workspace config"""
-    global DEFAULTS
+    global DEFAULTS  # pylint: disable=global-statement
 
     config = {
         key: value
@@ -419,18 +518,19 @@ def reset_defaults():
 
 
 def check_deprecated(kwargs):
+    """Check for deprecated arguments"""
     if kwargs.get("mate_scale") is not None:
         print("\nmate_scale is deprecated, use helper_scale instead\n")
         kwargs["helper_scale"] = kwargs["mate_scale"]
         del kwargs["mate_scale"]
 
-    if kwargs.get("reset_camera") == True:
+    if kwargs.get("reset_camera") is True:
         print(
             "\n'reset_camera=True' is deprecated, use 'reset_camera=Camera.RESET' instead\n"
         )
         kwargs["reset_camera"] = Camera.RESET
 
-    if kwargs.get("reset_camera") == False:
+    if kwargs.get("reset_camera") is False:
         print(
             "\n'reset_camera=False' is deprecated, use 'reset_camera=Camera.CENTER' instead\n"
         )
