@@ -1,3 +1,5 @@
+"""Show CAD objects in Visual Studio Code"""
+
 #
 # Copyright 2023 Bernhard Walter
 #
@@ -13,65 +15,70 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+
+import os
+import pathlib
 import re
+import time
+import types
+from enum import Enum
+from logging import Logger
 
-from ocp_tessellate import PartGroup
-from ocp_tessellate.convert import (
-    tessellate_group,
-    get_normal_len,
-    combined_bb,
-    to_assembly,
-    mp_get_results,
-    is_topods_shape,
-    is_cadquery,
-    is_vector,
-)
-from ocp_tessellate.utils import numpy_to_buffer_json, Timer, Color
-from ocp_tessellate.ocp_utils import (
-    is_vector,
-    is_topods_shape,
-    is_topods_compound,
-    is_cadquery,
-    is_cadquery_assembly,
-    is_cadquery_sketch,
-    is_build123d,
-    is_toploc_location,
-)
-
-from ocp_tessellate.mp_tessellator import init_pool, keymap, close_pool
+import ocp_tessellate.convert as oc
+from ocp_tessellate import OcpGroup
 from ocp_tessellate.cad_objects import (
-    OCP_PartGroup,
     OCP_Edges,
     OCP_Faces,
     OCP_Part,
+    OCP_PartGroup,
     OCP_Vertices,
 )
-from ocp_tessellate.convert import to_assembly
-import ocp_tessellate.convert as oc
+from ocp_tessellate.convert import (
+    combined_bb,
+    get_normal_len,
+    tessellate_group,
+    to_ocpgroup,
+)
+from ocp_tessellate.ocp_utils import (
+    is_build123d,
+    is_cadquery,
+    is_cadquery_assembly,
+    is_cadquery_sketch,
+    is_toploc_location,
+    is_topods_compound,
+    is_topods_shape,
+    is_vector,
+    is_wrapped,
+)
+from ocp_tessellate.utils import Color, Timer, numpy_to_buffer_json
 
-from .config import (
-    preset,
-    get_changed_config,
-    workspace_config,
-    combined_config,
-    get_default,
+from ocp_vscode.colors import BaseColorMap, get_colormap
+from ocp_vscode.comms import is_pytest, send_backend, send_command, send_data
+from ocp_vscode.config import (
     Camera,
     Collapse,
     check_deprecated,
+    combined_config,
+    get_changed_config,
+    get_default,
+    get_defaults,
+    preset,
+    workspace_config,
 )
-from .comms import send_backend, send_data
-from .colors import *
 
-from .persistence import modify_copyreg
-
-__all__ = ["show", "show_object", "reset_show", "show_all", "show_clear"]
+__all__ = [
+    "show",
+    "show_object",
+    "reset_show",
+    "show_all",
+    "show_clear",
+    "save_screenshot",
+]
 
 OBJECTS = {"objs": [], "names": [], "colors": [], "alphas": []}
 
 LAST_CALL = "other"
 
-modify_copyreg()
-
 
 def _tessellate(
     *cad_objs, names=None, colors=None, alphas=None, progress=None, **kwargs
@@ -85,26 +92,56 @@ def _tessellate(
 
     conf["reset_camera"] = reset_camera.value
 
-    collapse = conf.get("collapse", Collapse.LEAVES)
+    collapse = conf.get("collapse", Collapse.ROOT)
     conf["collapse"] = collapse.value
 
     if kwargs.get("default_facecolor") is not None:
         oc.FACE_COLOR = Color(kwargs["default_facecolor"]).percentage
         del kwargs["default_facecolor"]
     else:
-        oc.FACE_COLOR = Color(conf["default_facecolor"]).percentage
+        oc.FACE_COLOR = Color(conf.get("default_facecolor", oc.FACE_COLOR)).percentage
 
     if kwargs.get("default_thickedgecolor") is not None:
         oc.THICK_EDGE_COLOR = Color(kwargs["default_thickedgecolor"]).percentage
         del kwargs["default_thickedgecolor"]
     else:
-        oc.THICK_EDGE_COLOR = Color(conf["default_thickedgecolor"]).percentage
+        oc.THICK_EDGE_COLOR = Color(
+            conf.get("default_thickedgecolor", oc.THICK_EDGE_COLOR)
+        ).percentage
 
     if kwargs.get("default_vertexcolor") is not None:
         oc.VERTEX_COLOR = Color(kwargs["default_vertexcolor"]).percentage
         del kwargs["default_vertexcolor"]
     else:
-        oc.VERTEX_COLOR = Color(conf["default_vertexcolor"]).percentage
+        oc.VERTEX_COLOR = Color(
+            conf.get("default_vertexcolor", oc.VERTEX_COLOR)
+        ).percentage
+
+    # only use clipping settings when reset_camera is not RESET
+    if reset_camera == Camera.RESET or kwargs.get("reset_camera") == Camera.RESET:
+        clip_defaults = {
+            k: v for k, v in get_defaults().items() if k.startswith("clip")
+        }
+        if conf.get("clip_slider_0") is not None:
+            del conf["clip_slider_0"]
+        if conf.get("clip_slider_1") is not None:
+            del conf["clip_slider_1"]
+        if conf.get("clip_slider_2") is not None:
+            del conf["clip_slider_2"]
+        if conf.get("clip_normal_0") is not None:
+            conf["clip_normal_0"] = [-1, 0, 0]
+        if conf.get("clip_normal_1") is not None:
+            conf["clip_normal_1"] = [0, -1, 0]
+        if conf.get("clip_normal_2") is not None:
+            conf["clip_normal_2"] = [0, 0, -1]
+        if conf.get("clip_intersection") is not None:
+            conf["clip_intersection"] = False
+        if conf.get("clip_planes") is not None:
+            conf["clip_planes"] = False
+        if conf.get("clip_object_colors") is not None:
+            conf["clip_object_colors"] = False
+
+        conf.update(clip_defaults)
 
     timeit = preset("timeit", kwargs.get("timeit"))
 
@@ -114,9 +151,9 @@ def _tessellate(
     if progress is None:
         progress = Progress([c for c in "-+c"])
 
-    with Timer(timeit, "", "to_assembly", 1):
+    with Timer(timeit, "", "to_ocpgroup", 1):
         changed_config = get_changed_config()
-        part_group = to_assembly(
+        part_group, instances = to_ocpgroup(
             *cad_objs,
             names=names,
             colors=colors,
@@ -130,12 +167,13 @@ def _tessellate(
                 "default_color", changed_config.get("default_color")
             ),
             show_parent=kwargs.get("show_parent", changed_config.get("show_parent")),
+            show_sketch_local=kwargs.get(
+                "show_sketch_local", changed_config.get("show_sketch_local")
+            ),
             progress=progress,
         )
 
-        if len(part_group.objects) == 1 and isinstance(
-            part_group.objects[0], PartGroup
-        ):
+        if len(part_group.objects) == 1 and isinstance(part_group.objects[0], OcpGroup):
             part_group = part_group.objects[0]
 
     params = {
@@ -168,34 +206,18 @@ def _tessellate(
             print(f"Setting {k} can only be set in VSCode config")
 
         elif v is not None:
-            if k == "reset_camera" and params.get("_splash") == True:
+            if k == "reset_camera" and params.get("_splash") is True:
                 continue
             params[k] = v
 
-    parallel = preset("parallel", params.get("parallel"))
-    if parallel and not any(
-        [isinstance(obj, OCP_PartGroup) for obj in part_group.objects]
-    ):
-        print("parallel only works for assemblies, setting it to False")
-        parallel = False
-        params["parallel"] = False
-
     if kwargs.get("debug") is not None and kwargs["debug"]:
         print("\ntessellation parameters:\n", params)
 
     with Timer(timeit, "", "tessellate", 1):
-        if parallel:
-            init_pool()
-            keymap.reset()
-
-        instances, shapes, states, mapping = tessellate_group(
-            part_group, params, progress, params.get("timeit")
+        instances, shapes, mapping = tessellate_group(
+            part_group, instances, params, progress, params.get("timeit")
         )
 
-        if parallel:
-            instances, shapes = mp_get_results(instances, shapes, progress)
-            close_pool()
-
     params["normal_len"] = get_normal_len(
         preset("render_normals", params.get("render_normals")),
         shapes,
@@ -203,12 +225,18 @@ def _tessellate(
     )
 
     with Timer(timeit, "", "bb", 1):
-        bb = combined_bb(shapes).to_dict()
+        bb = combined_bb(shapes)
+        if bb is None:
+            bb = dict(
+                xmin=-1e-6, ymin=-1e-6, zmin=-1e-6, xmax=1e-6, ymax=1e-6, zmax=1e-6
+            )
+        else:
+            bb = bb.to_dict()
 
     # add global bounding box
     shapes["bb"] = bb
 
-    return instances, shapes, states, params, part_group.count_shapes(), mapping
+    return instances, shapes, params, part_group.count_shapes(), mapping
 
 
 def _convert(
@@ -224,7 +252,7 @@ def _convert(
     if progress is None:
         progress = Progress([c for c in "-+c"])
 
-    instances, shapes, states, config, count_shapes, mapping = _tessellate(
+    instances, shapes, config, count_shapes, mapping = _tessellate(
         *cad_objs,
         names=names,
         colors=colors,
@@ -245,9 +273,12 @@ def _convert(
         config["explode"] = kwargs["explode"]
 
     with Timer(timeit, "", "create data obj", 1):
+        if is_pytest():
+            return (instances, shapes, config, count_shapes), mapping
+
         return {
             "data": numpy_to_buffer_json(
-                dict(instances=instances, shapes=shapes, states=states)
+                dict(instances=instances, shapes=shapes),
             ),
             "type": "data",
             "config": config,
@@ -256,18 +287,22 @@ def _convert(
 
 
 class Progress:
+    """Progress indicator for tessellation"""
+
     def __init__(self, levels=None):
         if levels is None:
-            self.levels = ["+", "c", "-"]
+            self.levels = ["+c-*"]
         else:
             self.levels = levels
 
     def update(self, mark="+"):
+        """Update progress indicator"""
         if mark in self.levels:
             print(mark, end="", flush=True)
 
 
 def align_attrs(attr_list, length, default, tag, explode=True):
+    """Align attributes to the length of the cad_objs"""
     if attr_list is None:
         return [None] * length if explode else None
     elif len(attr_list) < length:
@@ -280,16 +315,16 @@ def align_attrs(attr_list, length, default, tag, explode=True):
         return attr_list
 
 
+# pylint: disable=unused-argument
 def show(
     *cad_objs,
     names=None,
     colors=None,
     alphas=None,
     port=None,
-    progress="-+c",
+    progress="-+*c",
     glass=None,
     tools=None,
-    measure_tools=None,
     tree_width=None,
     axes=None,
     axes0=None,
@@ -302,12 +337,22 @@ def show(
     collapse=None,
     explode=None,
     ticks=None,
+    center_grid=None,
     up=None,
     zoom=None,
     position=None,
     quaternion=None,
     target=None,
     reset_camera=None,
+    clip_slider_0=None,
+    clip_slider_1=None,
+    clip_slider_2=None,
+    clip_normal_0=None,
+    clip_normal_1=None,
+    clip_normal_2=None,
+    clip_intersection=None,
+    clip_planes=None,
+    clip_object_colors=None,
     pan_speed=None,
     rotate_speed=None,
     zoom_speed=None,
@@ -328,13 +373,14 @@ def show(
     render_mates=None,
     render_joints=None,
     show_parent=None,
-    parallel=None,
+    show_sketch_local=None,
     helper_scale=None,
     mate_scale=None,  # DEPRECATED
     debug=None,
     timeit=None,
     _force_in_debug=False,
 ):
+    # pylint: disable=line-too-long
     """Show CAD objects in Visual Studio Code
     Parameters
         cad_objs:                All cad objects that should be shown as positional parameters
@@ -344,13 +390,15 @@ def show(
         colors:                  List of colors for the cad_objs. Needs to have the same length as cad_objs
         alphas:                  List of alpha values for the cad_objs. Needs to have the same length as cad_objs
         port:                    The port the viewer listens to. Typically use 'set_port(port)' instead
-        progress:                Show progress of tessellation with None is no progress indicator. (default="-+c")
-                                 for object: "-": is reference, "+": gets tessellated, "c": from cache
+        progress:                Show progress of tessellation with None is no progress indicator. (default="-+*c")
+                                 for object: "-": is reference,
+                                             "+": gets tessellated with Python code,
+                                             "*": gets tessellated with native code,
+                                             "c": from cache
 
     Valid keywords to configure the viewer (**kwargs):
     - UI
         glass:                   Use glass mode where tree is an overlay over the cad object (default=False)
-        measure_tools:           Show measure tools (default=False)
         tools:                   Show tools (default=True)
         tree_width:              Width of the object tree (default=240)
 
@@ -367,8 +415,9 @@ def show(
                                  Collapse.ROOT: expand root only,
                                  Collapse.ALL: collapse all nodes,
                                  Collapse.NONE: expand all nodes
-                                 (default=Collapse.LEAVES)
+                                 (default=Collapse.ROOT)
         ticks:                   Hint for the number of ticks in both directions (default=10)
+        center_grid:             Center the grid at the origin or center of mass (default=False)
         up:                      Use z-axis ('Z') or y-axis ('Y') as up direction for the camera (default="Z")
         explode:                 Turn on explode mode (default=False)
 
@@ -381,6 +430,16 @@ def show(
                                  Camera.KEEP: Keep camera position, rotation, toom, and target
                                  (default=Camera.RESET)
 
+        clip_slider_0:           Setting of clipping slider 0 (default=None)
+        clip_slider_1:           Setting of clipping slider 1 (default=None)
+        clip_slider_2:           Setting of clipping slider 2 (default=None)
+        clip_normal_0:           Setting of clipping normal 0 (default=None)
+        clip_normal_1:           Setting of clipping normal 1 (default=None)
+        clip_normal_2:           Setting of clipping normal 2 (default=None)
+        clip_intersection:       Use clipping intersection mode (default=False)
+        clip_planes:             Show clipping plane helpers (default=False)
+        clip_object_colors:      Use object color for clipping caps (default=False)
+
         pan_speed:               Speed of mouse panning (default=1)
         rotate_speed:            Speed of mouse rotate (default=1)
         zoom_speed:              Speed of mouse zoom (default=1)
@@ -404,15 +463,32 @@ def show(
         render_normals:          Render normals (default=False)
         render_mates:            Render mates for MAssemblies (default=False)
         render_joints:           Render build123d joints (default=False)
-        parallel:                Tessellate objects in parallel (default=False)
-        show_parent:             Render parent of faces, edges or vertices as wireframe
+        show_parent:             Render parent of faces, edges or vertices as wireframe (default=False)
+        show_sketch_local:       In build123d show local sketch in addition to relocate sketch (default=True)
         helper_scale:            Scale of rendered helpers (locations, axis, mates for MAssemblies) (default=1)
 
     - Debug
         debug:                   Show debug statements to the VS Code browser console (default=False)
         timeit:                  Show timing information from level 0-3 (default=False)
     """
-    global LAST_CALL
+    global LAST_CALL  # pylint: disable=global-statement
+
+    if (
+        cad_objs is None
+        or len(cad_objs) == 0
+        or (
+            len(cad_objs) == 1
+            and (
+                cad_objs[0] is None
+                or (
+                    isinstance(cad_objs[0], (dict, list, tuple, set))
+                    and len(cad_objs[0]) == 0
+                )
+            )
+        )
+    ):
+        print("show: No CAD objects to show")
+        return
 
     kwargs = {
         k: v
@@ -438,17 +514,7 @@ def show(
 
     timeit = preset("timeit", timeit)
 
-    if measure_tools is None:
-        measure_tools = get_default("measure_tools")
-        if measure_tools is None:
-            conf = workspace_config()
-            measure_tools = conf["measure_tools"]
-    kwargs["measure_tools"] = measure_tools
-
-    if measure_tools and kwargs.get("collapse") is None:
-        kwargs["collapse"] = Collapse.ROOT
-
-    names = align_attrs(names, len(cad_objs), None, "names", explode=measure_tools)
+    names = align_attrs(names, len(cad_objs), None, "names")
 
     # Handle colormaps
 
@@ -465,19 +531,23 @@ def show(
         map_colors = [next(colormap) for _ in range(len(cad_objs))]
 
     for i in range(len(cad_objs)):
-        if isinstance(colors[i], str):
-            colors[i] = web_to_rgb(colors[i])
-        if colors[i] is None and map_colors is not None:
-            colors[i] = map_colors[i][:3]
-            if alphas[i] is None and len(map_colors[i]) == 4:
-                alphas[i] = map_colors[i][3]
-        elif colors[i] is not None:
-            if alphas[i] is None and len(colors[i]) == 4:
-                alphas[i] = colors[i][3]
-            colors[i] = colors[i][:3]
+        if colors[i] is None:
+            if map_colors is not None:
+                colors[i] = Color(map_colors[i][:3])
+                if alphas[i] is not None:
+                    colors[i].a = alphas[i]
+
+            if hasattr(cad_objs[i], "color") and cad_objs[i].color is not None:
+                # ensure that the explicitely given color is kept
+                colors[i] = Color(cad_objs[i].color)
+        else:
+            colors[i] = Color(colors[i])
+
+        if colors[i] is not None and alphas[i] is not None:
+            colors[i].a = alphas[i]
 
     if default_edgecolor is not None:
-        default_edgecolor = Color(default_edgecolor).web_color
+        default_edgecolor = Color(default_edgecolor)
 
     progress = Progress([] if progress is None else [c for c in progress])
 
@@ -496,19 +566,23 @@ def show(
         else:
             LAST_CALL = "other"
 
+    if is_pytest():
+        return t, mapping
+
     with Timer(timeit, "", "send"):
         send_data(t, port=port, timeit=timeit)
 
-    if measure_tools:
-        send_backend({"model": mapping}, port=port, timeit=timeit)
+    send_backend({"model": mapping}, port=port, timeit=timeit)
 
 
 def reset_show():
-    global OBJECTS
+    """Reset the stack of objects to be shown"""
+    global OBJECTS  # pylint: disable=global-statement
 
     OBJECTS = {"objs": [], "names": [], "colors": [], "alphas": []}
 
 
+# pylint: disable=too-many-locals,too-many-arguments
 def show_object(
     obj,
     name=None,
@@ -516,10 +590,9 @@ def show_object(
     parent=None,
     clear=False,
     port=None,
-    progress="-+c",
+    progress="-+*c",
     glass=None,
     tools=None,
-    measure_tools=None,
     tree_width=None,
     axes=None,
     axes0=None,
@@ -531,12 +604,22 @@ def show_object(
     orbit_control=None,
     collapse=None,
     ticks=None,
+    center_grid=None,
     up=None,
     zoom=None,
     position=None,
     quaternion=None,
     target=None,
     reset_camera=None,
+    clip_slider_0=None,
+    clip_slider_1=None,
+    clip_slider_2=None,
+    clip_normal_0=None,
+    clip_normal_1=None,
+    clip_normal_2=None,
+    clip_intersection=None,
+    clip_planes=None,
+    clip_object_colors=None,
     pan_speed=None,
     rotate_speed=None,
     zoom_speed=None,
@@ -556,13 +639,14 @@ def show_object(
     render_normals=None,
     render_mates=None,
     render_joints=None,
-    parallel=None,
     show_parent=None,
+    show_sketch_local=None,
     helper_scale=None,
     mate_scale=None,  # DEPRECATED
     debug=None,
     timeit=None,
 ):
+    # pylint: disable=line-too-long
     """Incrementally show CAD objects in Visual Studio Code
 
     Parameters:
@@ -576,13 +660,16 @@ def show_object(
         clear:                   In interactice mode, clear the stack of objects to be shown
                                  (typically used for the first object)
         port:                    The port the viewer listens to. Typically use 'set_port(port)' instead
-        progress:                Show progress of tessellation with None is no progress indicator. (default="-+c")
-                                 for object: "-": is reference, "+": gets tessellated, "c": from cache
+        progress:                Show progress of tessellation with None is no progress indicator. (default="-+*c")
+                                 for object: "-": is reference,
+                                             "+": gets tessellated with Python code,
+                                             "*": gets tessellated with native code,
+                                             "c": from cache
+
 
     Valid keywords to configure the viewer (**kwargs):
     - UI
         glass:                   Use glass mode where tree is an overlay over the cad object (default=False)
-        measure_tools:           Show measure tools (default=False)
         tools:                   Show tools (default=True)
         tree_width:              Width of the object tree (default=240)
 
@@ -599,8 +686,9 @@ def show_object(
                                  Collapse.ROOT: expand root only,
                                  Collapse.ALL: collapse all nodes,
                                  Collapse.NONE: expand all nodes
-                                 (default=Collapse.LEAVES)
+                                 (default=Collapse.ROOT)
         ticks:                   Hint for the number of ticks in both directions (default=10)
+        center_grid:             Center the grid at the origin or center of mass (default=False)
         up:                      Use z-axis ('Z') or y-axis ('Y') as up direction for the camera (default="Z")
 
         zoom:                    Zoom factor of view (default=1.0)
@@ -612,6 +700,16 @@ def show_object(
                                  Camera.KEEP: Keep camera position, rotation, toom, and target
                                  (default=Camera.RESET)
 
+        clip_slider_0:           Setting of clipping slider 0 (default=None)
+        clip_slider_1:           Setting of clipping slider 1 (default=None)
+        clip_slider_2:           Setting of clipping slider 2 (default=None)
+        clip_normal_0:           Setting of clipping normal 0 (default=[-1,0,0])
+        clip_normal_1:           Setting of clipping normal 1 (default=[0,-1,0])
+        clip_normal_2:           Setting of clipping normal 2 (default=[0,0,-1])
+        clip_intersection:       Use clipping intersection mode (default=[False])
+        clip_planes:             Show clipping plane helpers (default=False)
+        clip_object_colors:      Use object color for clipping caps (default=False)
+
         pan_speed:               Speed of mouse panning (default=1)
         rotate_speed:            Speed of mouse rotate (default=1)
         zoom_speed:              Speed of mouse zoom (default=1)
@@ -636,8 +734,8 @@ def show_object(
         render_normals:          Render normals (default=False)
         render_mates:            Render mates for MAssemblies (default=False)
         render_joints:           Render build123d joints (default=False)
-        parallel:                Tessellate objects in parallel (default=False)
-        show_parent:             Render parent of faces, edges or vertices as wireframe
+        show_parent:             Render parent of faces, edges or vertices as wireframe (default=False)
+        show_sketch_local:       In build123d show local sketch in addition to relocate sketch (default=True)
         helper_scale:            Scale of rendered helpers (locations, axis, mates for MAssemblies) (default=1)
 
     - Debug
@@ -652,8 +750,6 @@ def show_object(
         and k not in ["obj", "name", "options", "parent", "clear", "port", "progress"]
     }
 
-    global OBJECTS
-
     if clear:
         reset_show()
 
@@ -691,21 +787,29 @@ def show_object(
 
 
 def show_clear():
+    """Clear the viewer"""
     data = {
         "type": "clear",
     }
     send_data(data)
 
 
-def show_all(variables=None, exclude=None, force=False, **kwargs):
-    import inspect
+def show_all(
+    variables=None,
+    exclude=None,
+    classes=None,
+    _visual_debug=False,
+    **kwargs,
+):
+    """Show all variables in the current scope"""
+    import inspect  # pylint: disable=import-outside-toplevel
 
-    global LAST_CALL
+    global LAST_CALL  # pylint: disable=global-statement
 
-    if force:
+    if not _visual_debug:
         LAST_CALL = "other"
 
-    if LAST_CALL == "show":
+    if _visual_debug and LAST_CALL == "show":
         LAST_CALL = "other"
         print("\nSkip visual debug step after a show() command")
         return
@@ -721,15 +825,30 @@ def show_all(variables=None, exclude=None, force=False, **kwargs):
     names = []
     for name, obj in variables.items():
         if (
+            # ignore classes and ipython and jupyter variables
             isinstance(obj, type)
-            or name in ["_", "__", "___"]
-            or re.search("_\d+", name) is not None
+            or name in exclude + ["_", "__", "___", "_ih", "_oh", "_dh", "Out", "In"]
+            or name.startswith("__")
+            or re.search("_\\d+", name) is not None
+            # pylint: disable=protected-access
+            or (hasattr(obj, "_obj") and obj._obj is None)
+            or callable(obj)
+            or isinstance(obj, (int, float, str, bool, types.ModuleType))
+            or obj is None
+            or isinstance(obj, Enum)
+            or (
+                obj.__class__.__name__ == "_Feature"
+                and obj.__class__.__module__ == "__future__"
+            )
+            or isinstance(obj, Logger)
         ):
-            continue  # ignore classes and jupyter variables
+            continue
 
-        if name not in exclude:
-            if hasattr(obj, "_obj") and obj._obj is None:
-                continue
+        if hasattr(obj, "area") and obj.area > 1e99:  # inifinite face
+            print(f"infinite face {name} skipped")
+            continue
+
+        if classes is None or isinstance(obj, tuple(classes)):
 
             if hasattr(obj, "locations") and hasattr(obj, "local_locations"):
                 obj = obj.locations
@@ -750,16 +869,12 @@ def show_all(variables=None, exclude=None, force=False, **kwargs):
                 or is_cadquery(obj)
                 or is_build123d(obj)
                 or is_cadquery_assembly(obj)
-                or (
-                    isinstance(obj, (list, tuple))
-                    and len(obj) > 0
-                    and hasattr(obj[0], "wrapped")
-                )
                 or (
                     hasattr(obj, "wrapped")
                     and hasattr(obj, "position")
                     and hasattr(obj, "direction")
                 )
+                or isinstance(obj, (list, tuple, dict))
             ):
                 objects.append(obj)
                 names.append(name)
@@ -772,18 +887,54 @@ def show_all(variables=None, exclude=None, force=False, **kwargs):
                 names.append(name)
 
             elif is_cadquery_sketch(obj):
-                pg = to_assembly([obj], names=[name])
+                pg = to_ocpgroup([obj], names=[name])
                 pg.name = name
                 objects.append(pg)
                 names.append(name)
 
+            else:
+                print(
+                    f"show_all: Type {type(obj)} for name {name} cannot be visualized"
+                )
+
     if len(objects) > 0:
-        show(
-            *objects,
-            names=names,
-            collapse=Collapse.ROOT,
-            _force_in_debug=True,
-            **kwargs,
-        )
+        try:
+            result = show(
+                *objects,
+                names=names,
+                collapse=Collapse.ROOT,
+                _force_in_debug=_visual_debug,
+                **kwargs,
+            )
+            if is_pytest():
+                return result
+        except Exception as ex:  # pylint: disable=broad-exception-caught
+            print("show_all:", ex)
     else:
+        if is_pytest():
+            return None
         show_clear()
+
+
+def save_screenshot(filename, port=None):
+    """Save a screenshot of the current view"""
+    if not filename.startswith(os.sep):
+        prefix = pathlib.Path(".").absolute()
+        full_path = str(prefix / filename)
+    else:
+        full_path = filename
+    p = pathlib.Path(full_path)
+    mtime = p.stat().st_mtime if p.exists() else 0
+
+    send_command({"type": "screenshot", "filename": f"{full_path}"}, port=port)
+
+    done = False
+    for i in range(20):
+        if p.exists() and p.stat().st_mtime > mtime:
+            print("Screenshot saved to ", full_path)
+            done = True
+            break
+        time.sleep(0.1)
+
+    if not done:
+        "Screenshot not found in 2 seconds, aborting"
